#
# Rawhide Acceptance Test Suite - common functions
#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>

import optparse
import util, config

# Map keyname (as used in the RATS tests) to testid (as used by israwhidebroken)
testdict = {'repodata':1, 'comps':2, 'critpath_deps':3, 'critpath_integrity':4,
            'images':5, 'kernel_boot':6, 'load_stage2':7, 'diskprobe':8,
            'package_install':9, 'bootloader':10}

def report_results(testlist, keyvals, url=None, username='', password=''):
    '''Use fedora.client to report results to israwhidebroken at the URL given,
    using the FAS username/password given.
    May raise fedora.client.{ServerError,AuthError,AppError} or pycurl.error.
    '''
    from fedora.client import BaseClient
    irb = BaseClient(url, username=username, password=password)
    # Set up the dict of tree info so we can find the tree ID
    timestamp = keyvals.get('repodata_time') or keyvals.get('tree_time') or None
    if timestamp:
        timestamp = int(timestamp)
    treeinfo = {'arch':util.get_basearch(),
                'compose_id':util.timestamp_to_compose_id(timestamp)}
    r = irb.send_request('/get_trees', req_params=treeinfo)
    # Update the tree if it exists, and add it if it doesn't
    for f in ('repodata_time', 'tree_time'):
        if f in keyvals:
            treeinfo[f] = keyvals[f]
    if len(r['trees']) > 0:
        # NOTE: we're somewhat arbitrarily updating the newest matching tree
        treeinfo['treeid'] = max([t['id'] for t in r['trees']])
        r = irb.send_request('/update_tree', auth=True, req_params=treeinfo)
    else:
        r = irb.send_request('/add_tree', auth=True, req_params=treeinfo)
    # Ok, we got the tree, now add some results for it
    treeid = r['tree']['id']
    for testname in testlist:
        params = {'treeid':treeid,
                  'testid':testdict[testname],
                  'result':(keyvals.get(testname,'') == 'OK') and 1 or 0}
        # TODO: add 'detail_url':"link to this job's results"
        # TODO: handle other statuses (e.g. WARN)
        irb.send_request('/add_result', auth=True, req_params=params)

def get_parser():

    parser = optparse.OptionParser(usage="%prog [options] TREEURL")

    archlist = ['i386', 'x86_64', 'ppc']
    parser.add_option("-a", "--arch", action="store", type="choice",
        choices=archlist, default=util.get_basearch(),
        help="target architecture (%default)")
    parser.add_option("-s", "--scratchdir", action="store",
        default=config.scratchdir,
        help="scratch dir for downloaded/generated files (%default)")
    parser.add_option("-l", "--logdir", action="store",
        default=config.logdir,
        help="write logs to this dir (%default)")
    parser.add_option("-b", "--bindir", action="store",
        default=config.bindir,
        help="binary dir - contains minimon, ks.cfg, etc. (%default)")
    return parser

def parse_args(parser=None):
    # If no parser supplied, get one
    if parser is None:
        parser = get_parser()

    (opt, args) = parser.parse_args()

    # Validate arguments
    if len(args) != 1:
        parser.error("incorrect number of arguments")

    # Process TREEURL
    baseurl = args[0]
    if not baseurl.endswith('/'):
        baseurl = baseurl + '/'

    util.grabber.opts.prefix = baseurl
    config.set_scratchdir(opt.scratchdir)
    config.logdir = opt.logdir
    config.bindir = opt.bindir
    return (opt, baseurl)

