#
# virtguest.py - simple library for creating & monitoring simple virt guests
#
# Copyright 2009, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Will Woods <wwoods@redhat.com>
#
# TODO:
# - Override kickstart contents/location
#   - Overriding contents allows for crazy partitioning magic
# - Pass in an updates.img
#   - Via URL, local (virtual) media
# - Handle iso install source (DVD)
# - Handle multiple iso install source (CD set)
# - Handle NFS / hdiso setup?
# - iSCSI? D:
# - Maybe convert VirtGuest to SimpleVirtGuest, and have a more complex
#   parent VirtGuest class that implements all that junk

import os
import sys
import time
import select
from subprocess import *
from xml.etree import cElementTree as ElementTree

def findhostnetinfo(network, item, attr):
    p = Popen(['virsh', 'net-dumpxml', network], stdout=PIPE)
    (netxml, dummy) = p.communicate()
    assert p.returncode == 0
    root = ElementTree.fromstring(netxml)
    element = root.find(item)
    if element is None:
        return None
    for k, v in element.items():
        if k == attr:
            return v

def findhostip(network='default'):
    return findhostnetinfo(network, 'ip', 'address')

def findhostbridge(network='default'):
    return findhostnetinfo(network, 'bridge', 'name')

class VirtGuest(object):
    '''dumb little class to hold data about virt guests'''
    def __init__(self, name, disk=None, wait=60, network='default'):
        self.name = name
        self.disk = disk or '/var/lib/libvirt/images/%s.img' % name
        self.diskcount = 1
        self.wait = wait
        self.network = network
        self.birthdate = 0
        self.showdots = False # TODO set to true if stdout is a tty?
        self.active = False
        self.tty = None
        self.console = None
        self.pollobj = select.poll()
        self.os_variant = 'fedora11'

    def age(self):
        '''return the age of this guest in seconds since object creation'''
        return (time.time() - self.birthdate)

    def findtty(self):
        p = Popen(['virsh', 'ttyconsole', self.name], stdout=PIPE)
        (output, dummy) = p.communicate()
        tty = output.strip()
        assert os.path.exists(tty)
        self.tty = tty
        return tty

    def opentty(self):
        if self.tty is None:
            self.findtty()
        self.console = open(self.tty)
        self.pollobj.register(self.console, select.POLLIN)
        return self.tty

    def closetty(self):
        self.pollobj.unregister(self.console)
        self.console.close()

    def findhostip(self):
        return findhostip(self.network)

    def findhostbridge(self):
        return findhostbridge(self.network)

    def poll(self, wait=None):
        if wait is None:
            wait = self.wait
        return self.pollobj.poll(wait*1000)

    def poll_and_read(self, wait=None, blocksize=4096):
        '''Poll for incoming data, read it in blocksize blocks, and return it'''
        events = self.poll(wait)
        if not events:
            return None
        data = ''
        for (fd, event) in events:
            assert fd == self.console.fileno()
            if (event & (select.POLLIN | select.POLLPRI)):
                self.dot('r')
                data += os.read(fd, blocksize)
            else:
                self.dot('?')
        return data

    def dot(self, dot='.', out=sys.stdout):
        if self.showdots:
            out.write(dot)
            out.flush()

    def read_until(self, condition, readtimeout=60, max_wait=600, logfd=None):
        '''Read data from the guest's console until the condition becomes true
        one of the timeouts is reached.
        condition should be a callable that takes one argument - accumulated
        data read from the guest's serial console up to that point. If
        condition evaluates to True, all accumulated data will be returned.
        readtimeout is the inactivity timeout in seconds - if no data is sent
        for that amount of time, None will be returned.
        max_wait is the maximum amount of time (in seconds) to wait for the
        condition to become true - even if there's constant output.
        logfd, should be an open, writeable file descriptor. All data read from
        the serial console will be written there.
        examples:
        read_until(lambda x: 'this message indicates success' in x)
        read_until(log_parser_func, logfd=open("serial.log", "w"))
        '''
        data = ""
        start = time.time()
        while (time.time() - start) < max_wait:
            self.dot('p')
            newdata = self.poll_and_read(readtimeout)
            if newdata is None:
                break
            self.dot('w')
            if logfd is not None:
                logfd.write(newdata)
                logfd.flush()
            data += newdata
            if condition(data):
                self.dot(':D\n')
                return data
        self.dot(':/\n')

    def create(self, location, ram=768, disksize=8, extra_args='', serial=True):
        '''
        Create a virtual guest by calling `virt-install` with appropriate arguments

        @param location: A directory containing install images
        @param ram: Size of memory (in megabytes) to allocate for guest
        @param disksize: Size of disk (in gigabytes) to allocate for guest
        @param extra_args: Additional boot arguments for the guest
        @raises RuntimeError: If the return value from `virt-install` is non-zero
        '''
        if serial:
            extra_args += ' serial console=ttyS0'
        virt_args = [
            '--name', self.name,
            '--ram', str(ram),
            '--vcpus', '1',
            '--os-type', 'linux',
            '--os-variant', self.os_variant,
            '--extra-args', '%s ks=file:/ks.cfg' % extra_args,
            '--disk', 'path=%s,size=%s,sparse=false' % (self.disk, disksize),
            '--network', 'network:%s' % self.network,
            '--location', location,
            '--accelerate',
            '--nographics',
            '--noautoconsole']
        if self.diskcount > 1:
            for n in range(1, self.diskcount): # e.g. 5 -> [1,2,3,4]
                virt_args += ['--disk',
                  'path=%s.%d,size=%s,sparse=false' % (self.disk, n, disksize)]
        self.create_command = ['virt-install'] + virt_args
        retval = call(self.create_command)
        if retval != 0:
            self.delete()
            raise RuntimeError, "virt-install failed: %i" % retval
        self.birthdate = time.time()
        self.active = True

    def delete(self):
        if self.name and self.disk:
            call(["virsh", "destroy", self.name])
            call(["virsh", "undefine", self.name])
            call(["virsh", "vol-delete", self.disk])
            self.active = False

    def exists(self):
        devnull = open(os.devnull, "w+")
        retval = call(["virsh", "dominfo", self.name],
                      stdout=devnull, stderr=devnull)
        return retval == 0
