#!/bin/bash

usage() {
    cat <<EOF
Usage: $0 [options]

Options:
   -h                   show this help message and exit
   -r RELEASE           Value to use for \$releasever (example: epel-5,
                        fedora-14, fedora-rawhide)
   -a ARCH              Value used for \$basearch
   -b BRANCH            Specify anaconda git branch (default: master)
   -d RESULTDIR         Directory to store results (default: \$PWD)
   -m TEMPLATE          Mock template to use (default: mock.cfg.in)
   -t TMPDIR            Temporary directory to use (default: /tmp)
EOF
    exit 1
}

error() {
    echo "Error: $1"
    exit 1
}

detect_releasever() {
    RELEASEVER=""
    # Guess releasever based on current system release
    if [ -f /etc/fedora-release ]; then
        VER=$(gawk '{print $3}' /etc/fedora-release)
        RELEASEVER="fedora-$VER"
    elif [ -f /etc/redhat-release -a ! -L /etc/redhat-release ]; then
        VER=$(gawk '{print $5}' /etc/redhat-release)
        case $VER in 
            4* ) VER=4 ;;
            5* ) VER=5 ;;
            6* ) VER=6 ;;
            * ) VER=6 ;;
        esac
        RELEASEVER="epel-$VER"
    fi
    echo $RELEASEVER
}

cleanup() {
    rm ${SETUP}
    rm ${MOCKDIR}/${MOCKCFG}.cfg
    exit 1
}

# Default values
BRANCH="master"
BASEARCH=$(uname -i)
RELEASEVER=$(detect_releasever)
RESULTSDIR=${PWD}/results/$(date +%Y%m%d)
TEMPLATE=${PWD}/mock.cfg.in
TMPDIR=/tmp
MOCKDIR=/etc/mock

# Process arguments
while getopts  "r:b:a:d:m:t:h" option
do
  case $option in
    a ) BASEARCH=$OPTARG ;;
    r ) RELEASEVER=$OPTARG ;;
    b ) BRANCH=$OPTARG ;;
    d ) RESULTSDIR=$OPTARG ;;
    m ) TEMPLATE=$OPTARG ;;
    t ) TMPDIR=$OPTARG ;;
    \?|h ) usage ;;
    * ) usage ;;
  esac
done

# Sanitize input arguments
test -f $TEMPLATE || error "Template not found: $TEMPLATE"
test -d $MOCKDIR || error "Directory $MOCKDIR not found.  Is mock installed?"
# FIXME - sanitize basearch
# FIXME - sanitize releasever

SETUP=$(mktemp ${TMPDIR}/checkbot-setup.XXXXXX)

# Prepare mock template
MOCKCFG="checkbot-$RELEASEVER-$BASEARCH"
sed -e "s|{basearch}|$BASEARCH|;s|{releasever}|$RELEASEVER|" $TEMPLATE > $MOCKDIR/$MOCKCFG.cfg
if [ "$?" -ne 0 ]; then
    error "Failed while building mock configuration file"
fi

# Ensure cleanup is called on exit
trap cleanup SIGTERM

# Set up the anaconda source tree in preparation for pylint.
cat > ${SETUP} << EOF
#!/bin/sh
if [ -d /anaconda ]; then
    git fetch && git rebase origin && git checkout -b ${BRANCH} origin/${BRANCH}
else
    git clone -b ${BRANCH} git://git.fedorahosted.org/git/anaconda.git
fi

cd anaconda && yum --skip-broken install \$(grep Requires: anaconda.spec.in | cut -d' ' -f2)
EOF

MOCK="mock -r $MOCKCFG"
${MOCK} --init
${MOCK} --install git libtool pylint xorg-x11-server-Xvfb xorg-x11-xauth
${MOCK} --copyin ${SETUP} /tmp/setup.sh
${MOCK} --chroot "bash /tmp/setup.sh"
${MOCK} --chroot "( cd /anaconda && ./autogen.sh && ./configure && make ) 2>&1 | tee /tmp/build.log"
${MOCK} --chroot "cd /anaconda && xvfb-run tests/pylint/runpylint.sh"
${MOCK} --chroot "( cd /anaconda && make unittest ) 2>&1 | tee /tmp/unittest.log"
${MOCK} --chroot "( cd /anaconda && make scratch && rpmbuild -tb anaconda*.tar.bz2 ) 2>&1 | tee /tmp/rpmbuild.log"

# Report the results.  We do all the copies individually in case one of the files doesn't exist.
mkdir -p ${RESULTSDIR}
${MOCK} --copyout /tmp/build.log ${RESULTSDIR}
${MOCK} --copyout /anaconda/pylint-log ${RESULTSDIR}
${MOCK} --copyout /tmp/unittest.log ${RESULTSDIR}
${MOCK} --copyout /tmp/rpmbuild.log ${RESULTSDIR}
cp /var/lib/mock/${MOCKCFG}/result/root.log ${RESULTSDIR}

# And clean up.
cleanup
